/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package org.gvsig.graph.mt;

import java.io.File;
import java.util.ArrayList;

import org.gvsig.exceptions.BaseException;
import org.gvsig.graph.core.GraphException;
import org.gvsig.graph.core.GvFlag;
import org.gvsig.graph.core.Network;
import org.gvsig.graph.core.NetworkUtils;
import org.gvsig.graph.mt.gui.AssignmentEqControlPanel;
import org.gvsig.graph.mt.solvers.AssignmentEqTask;

import com.iver.andami.PluginServices;
import com.iver.andami.plugins.Extension;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.drivers.IFeatureIterator;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.ReadableVectorial;
import com.iver.cit.gvsig.fmap.layers.SingleLayerIterator;
import com.iver.cit.gvsig.project.documents.view.gui.View;
import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.Point;

/**
 * @author Tonatiuh Miguel (tonatiuh.sit@gmail.com)
 *
 */
public class AssignmentEquilibriumExtension extends Extension{

	private static MapControl mapControl;
	
	public void initialize() {
		PluginServices.getIconTheme().registerDefault(
				"assignment",
				this.getClass().getClassLoader().getResource("images/assignment.png")
			);
	}

	public void execute(String actionCommand) {
		
		View v = (View) PluginServices.getMDIManager().getActiveWindow();
		MapContext map = v.getMapControl().getMapContext();
		SingleLayerIterator it = new SingleLayerIterator(map.getLayers());

		if (actionCommand.equals("MTEQUILIBRIUM")) {
			while (it.hasNext())
			{
				FLayer aux = it.next();
				if (!aux.isActive())
					continue;
				Network net = (Network) aux.getProperty("network");

				if ( net != null)
				{
					AssignmentEqControlPanel ctrlDlg = new AssignmentEqControlPanel();
					try {
						ctrlDlg.setMapContext(map);
						PluginServices.getMDIManager().addWindow(ctrlDlg);
						if (ctrlDlg.isOkPressed()) {
							if (net.getLayer().getISpatialIndex() == null) {
								System.out.println("Calculando ndice espacial (QuadTree, que es ms rpido)...");
								net.getLayer().setISpatialIndex(
										NetworkUtils.createJtsQuadtree(net.getLayer()));
								System.out.println("ndice espacial calculado.");
							}
							
							FLyrVect layerOrigins = ctrlDlg.getOriginsLayer();
							FLyrVect layerDestinations = ctrlDlg.getDestinationsLayer();
							double tolerance = ctrlDlg.getTolerance();
							ArrayList<Double> costFreeFlow = ctrlDlg.getCostFF();
							ArrayList<Double> serviceLevel = ctrlDlg.getServiceLevel();
							double epsilon = ctrlDlg.getEpsilon();
							File tripMatrixFile = new File(ctrlDlg.getTripMatrix());
							File generatedFile = new File(ctrlDlg.getPathGenerateFile());
							
							GvFlag[] originFlags = this.putFlagsToNodeOnNetwork(layerOrigins, net, tolerance);
							GvFlag[] destinationFlags = this.putFlagsToNodeOnNetwork(layerDestinations, net, tolerance);
							
							AssignmentEqTask task = new AssignmentEqTask(net,
									originFlags, destinationFlags,
									costFreeFlow, serviceLevel, epsilon,
									tripMatrixFile, generatedFile);
							PluginServices.cancelableBackgroundExecution(task);
							
							return;
						}
					}catch (BaseException e) {
						e.printStackTrace();
					}
				}
			}
		}
		
	}

	public boolean isEnabled() {
		Network net = getNetwork();
		if ( net != null)
		{
			return true;
		}
		else 
			return false;
	}

	public boolean isVisible() {
		IWindow f = PluginServices.getMDIManager().getActiveWindow();
		if (f == null) {
		    return false;
		}
		if (f instanceof View) {
			return true;
		}
		return false;
	}

	public static Network getNetwork(){
		IWindow window = PluginServices.getMDIManager().getActiveWindow();
		if (window instanceof View)
		{
			View v = (View) window;
	        mapControl = v.getMapControl();
			MapContext map = mapControl.getMapContext();
			
			SingleLayerIterator it = new SingleLayerIterator(map.getLayers());
			while (it.hasNext())
			{
				FLayer aux = it.next();
				if (!aux.isActive())
					continue;
				Network net = (Network) aux.getProperty("network");
				return net;
			}
		}
		return null;
	}
	
	public MapControl getMapControl(){
		return mapControl;
	}
	
	private GvFlag[] putFlagsToNodeOnNetwork(FLyrVect layer, Network net,
			double tolerance) throws BaseException {
		ReadableVectorial reader = layer.getSource();
		reader.start();
		IFeatureIterator it = reader.getFeatureIterator();
		int i = 0;
		ArrayList<GvFlag> flags = new ArrayList<GvFlag>();
		while (it.hasNext()) {
			IFeature feat = it.next();
			Geometry geo = feat.getGeometry().toJTSGeometry();
			if (!((geo instanceof Point) || (geo instanceof MultiPoint)))
				continue;

			Coordinate[] coords = geo.getCoordinates();
			for (int j = 0; j < coords.length; j++) {
				GvFlag flag = net.addFlagToNode(coords[j].x, coords[j].y, tolerance);
				if (flag == null) {
					GraphException e = new GraphException("Punto " + i
							+ " fuera de la red. Tolerancia=" + tolerance);
					e.setCode(GraphException.FLAG_OUT_NETWORK);
					throw e;
				}
				System.out.println("Situando flag " + i + " de la capa " + layer.getName() + " en idArc=" + flag.getIdArc());
				flags.add(flag);
				flag.getProperties().put("rec", new Integer(i));
				break;
			}
			i++;
		}
		it.closeIterator();
		reader.stop();
		return flags.toArray(new GvFlag[0]);
	}
}
