/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package org.gvsig.graph.mt.gui;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import com.hardcode.gdbms.driver.exceptions.ReadDriverException;
import com.hardcode.gdbms.engine.values.ValueWriter;
import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.WindowInfo;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.LayersIterator;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;

import javax.swing.SwingConstants;

import org.gvsig.exceptions.BaseException;
import org.gvsig.fmap.util.LayerListCellRenderer;



/**
 * @author Tonatiuh Miguel (tonatiuh.sit@gmail.com)
 *
 */
public class AssignmentEqControlPanel extends JPanel implements IWindow{
	
	private JPanel layersPanel, fieldsPanel, filesPanel, buttonsPanel, contentPanel;
	
	private JLabel lblOrigins, lblDestinations, lblTolerance, lblMeters;
	private JLabel lblCostFF, lblServiceL, lblEpsilon, lblMasMenos;
	private JLabel lblTripMatrix, lblCreateFile;
	
	private JComboBox cmbOrigins, cmbDestinations, cmbCostFF, cmbServiceL;
	
	private JTextField txtEpsilon, txtTripMatrix, txtPathGenerateFile, txtTolerance;
	
	private JButton btnTripMatrix, btnCreateFile, btnCancel, btnRun;
	
	private MapContext mapContext;
	private WindowInfo wi;
	private boolean okPressed = false;
	
	private FLyrVect currentLyr;
	private ArrayList<Double> costFF;
	private ArrayList<Double> serviceLevel;
	
	private DefaultComboBoxModel cmbOriginsModel, cmbDestinationsModel;
	private DefaultComboBoxModel cmbCostFFModel, cmbServiceLModel;
	
	public static void main(String[] args) throws ClassNotFoundException,
			InstantiationException, IllegalAccessException {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());		
		} catch (UnsupportedLookAndFeelException e) {
			e.printStackTrace();
		}
		AssignmentEqControlPanel panel = new AssignmentEqControlPanel();
		JFrame test = new JFrame();
		
		test.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		test.getContentPane().add(panel);
		test.setBounds(panel.getBounds());
		test.setVisible(true);
	}
	
	public AssignmentEqControlPanel() {
		initComponents();
	}
	
	private void initComponents(){
		
		//Layers section
		lblOrigins = new JLabel((_T("Origins")+":"),JLabel.RIGHT);
		lblOrigins.setBounds(15, 30, 145, 22);
		lblDestinations = new JLabel((_T("Destinations")+":"),JLabel.RIGHT);
		lblDestinations.setBounds(15, 70, 145, 22);
		lblTolerance = new JLabel((_T("Tolerance")+":"),JLabel.RIGHT);
		lblTolerance.setBounds(15, 110, 145, 22);
		lblMeters = new JLabel((_T("meters")),JLabel.LEFT);
		lblMeters.setBounds(310, 110, 100, 22);
		cmbOrigins = new JComboBox();
		cmbOrigins.setBounds(170, 30, 240, 22);
		cmbDestinations = new JComboBox();
		cmbDestinations.setBounds(170, 70, 240, 22);
		txtTolerance = new JTextField(50);
		txtTolerance.setText("50");
		txtTolerance.setBounds(170, 110, 130, 22);
		
		//Fields section
		lblCostFF = new JLabel((_T("Cost_free_flow")+":"),JLabel.RIGHT);
		lblCostFF.setBounds(15, 30, 145, 22);
		lblServiceL = new JLabel((_T("Service_level")+":"),JLabel.RIGHT);
		lblServiceL.setBounds(15, 70, 145, 22);
		lblEpsilon = new JLabel((_T("Epsilon")+":"),JLabel.RIGHT);
		lblEpsilon.setBounds(15, 110, 145, 22);
		lblMasMenos = new JLabel((_T("+-")), SwingConstants.CENTER);
		lblMasMenos.setBounds(170, 110, 15, 22);
		cmbCostFF = new JComboBox();
		cmbCostFF.setBounds(170, 30, 240, 22);
		cmbServiceL = new JComboBox();
		cmbServiceL.setBounds(170, 70, 240, 22);

		cmbCostFF.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int selectedFieldIndex = 0;
				try {
					String selectedField = cmbCostFF.getSelectedItem().toString();
					selectedFieldIndex = currentLyr.getRecordset().getFieldIndexByName(selectedField);
				} catch (ReadDriverException ex){
					ex.printStackTrace();
				}
				
				costFF = getValuesCostFF(selectedFieldIndex);
			}
		});
		
		cmbServiceL.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int selectedFieldIndex = 0;
				try {
					String selectedField = cmbServiceL.getSelectedItem().toString();
					selectedFieldIndex = currentLyr.getRecordset().getFieldIndexByName(selectedField);
				} catch (ReadDriverException ex) {
					ex.printStackTrace();
				}
				
				serviceLevel = getValuesServiceLevel(selectedFieldIndex);
			}
		});
		
		txtEpsilon = new JTextField();
		txtEpsilon.setText("0.01");
		txtEpsilon.setBounds(185, 110, 115, 22);
		
		//Files section
		lblTripMatrix = new JLabel((_T("Trip_matrix")+":"),JLabel.RIGHT);
		lblTripMatrix.setLocation(15, 30);
		lblTripMatrix.setSize(145, 22);
		lblCreateFile = new JLabel((_T("Create_file")+":"),JLabel.RIGHT);
		lblCreateFile.setLocation(15, 70);
		lblCreateFile.setSize(145, 22);
		txtTripMatrix = new JTextField();
		txtTripMatrix.setLocation(170, 30);
		txtTripMatrix.setSize(240, 22);
		txtPathGenerateFile = new JTextField();
		txtPathGenerateFile.setLocation(170, 70);
		txtPathGenerateFile.setSize(240, 22);
		
		btnTripMatrix = new JButton();
		btnTripMatrix.setText("...");
		btnTripMatrix.setLocation(420, 30);
		btnTripMatrix.setSize(40, 22);
		btnTripMatrix.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser dlgTM = new JFileChooser();
				if (dlgTM.showOpenDialog((Component) PluginServices.getMainFrame()) == JFileChooser.APPROVE_OPTION) 
				{
					txtTripMatrix.setText(dlgTM.getSelectedFile().getPath());
				}
			}
		});
		
		btnCreateFile = new JButton();
		btnCreateFile.setText("...");
		btnCreateFile.setLocation(420, 70);
		btnCreateFile.setSize(40, 22);
		btnCreateFile.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				JFileChooser dlgCF = new JFileChooser();
				if (dlgCF.showSaveDialog((Component) PluginServices.getMainFrame()) == JFileChooser.APPROVE_OPTION)
				{
					txtPathGenerateFile.setText(dlgCF.getSelectedFile().getPath());
				}
			}
		});
		
		//Buttons section
		btnCancel = new JButton(_T("Cancel"));
		btnCancel.setBounds(90, 11, 135, 25);
		btnCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				btnCancelActionPerformed(evt);
			}
		});
		
		btnRun = new JButton(_T("Run"));
		btnRun.setBounds(255, 11, 135, 25);
		btnRun.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent evt) {
				btnRunActionPerformed(evt);
			}
		});
				
		//JPanels section
		layersPanel = new JPanel(null);
		layersPanel.setBounds(10, 11, 480, 150);
		layersPanel.setBorder(BorderFactory.createTitledBorder(_T("Layers")));
		layersPanel.setLayout(null);
		layersPanel.add(lblOrigins);
		layersPanel.add(lblDestinations);
		layersPanel.add(lblTolerance);
		layersPanel.add(lblMeters);
		layersPanel.add(cmbOrigins);
		layersPanel.add(cmbDestinations);
		layersPanel.add(txtTolerance);
		
		fieldsPanel = new JPanel(null);
		fieldsPanel.setBounds(10, 172, 480, 150);
		fieldsPanel.setBorder(BorderFactory.createTitledBorder(_T("Fields")));
		fieldsPanel.setLayout(null);
		fieldsPanel.add(lblCostFF);
		fieldsPanel.add(lblServiceL);
		fieldsPanel.add(lblEpsilon);
		fieldsPanel.add(lblMasMenos);
		fieldsPanel.add(cmbCostFF);
		fieldsPanel.add(cmbServiceL);
		fieldsPanel.add(txtEpsilon);
		
		filesPanel = new JPanel(null);
		filesPanel.setBounds(10, 333, 480, 100);
		filesPanel.setBorder(BorderFactory.createTitledBorder(_T("Files")));
		filesPanel.add(lblTripMatrix, null);
		filesPanel.add(lblCreateFile, null);
		filesPanel.add(txtTripMatrix, null);
		filesPanel.add(txtPathGenerateFile, null);
		filesPanel.add(btnTripMatrix, null);
		filesPanel.add(btnCreateFile, null);
		
		buttonsPanel = new JPanel();
		buttonsPanel.setLayout(null);		
		buttonsPanel.setBounds(10, 444, 480, 50);
		buttonsPanel.add(btnCancel);
		buttonsPanel.add(btnRun);
		
		contentPanel = new JPanel(null);
		contentPanel.setSize(500, 500);
		contentPanel.add(layersPanel);
		contentPanel.add(fieldsPanel);
		contentPanel.add(filesPanel);
		contentPanel.add(buttonsPanel);
		
		this.setLayout(null);
		this.add(contentPanel);
		
	}
	
	public void setMapContext(MapContext mapContext) throws BaseException {
		this.mapContext = mapContext;
		FLayers layers = mapContext.getLayers();
		LayersIterator it = new LayersIterator(layers);
		Vector<FLyrVect> arrayLayers = new Vector<FLyrVect>();
		Vector<String> arrayFields = new Vector<String>();
		while (it.hasNext()) {
			FLayer lyr = it.nextLayer();
			if (!lyr.isAvailable())
				continue;
			if (lyr instanceof FLyrVect) {
				FLyrVect lyrVect = (FLyrVect) lyr;
				if ((lyrVect.getShapeType() == FShape.POINT)
						|| (lyrVect.getShapeType() == FShape.MULTIPOINT)) {
					FLyrVect lyrVectPoint = (FLyrVect) lyrVect;
					arrayLayers.add(lyrVectPoint);
				}					
				if ((lyrVect.getShapeType() == FShape.LINE) && (lyrVect.isActive())) {
					this.currentLyr = (FLyrVect) lyrVect;
					try {
						String[] aux = currentLyr.getRecordset().getFieldNames();
						for (int i = 0; i < aux.length; i++) {
							switch (currentLyr.getRecordset().getFieldType(i)) {
							case Types.BIGINT:
							case Types.DECIMAL:
							case Types.DOUBLE:
							case Types.FLOAT:
							case Types.INTEGER:
							case Types.NUMERIC:
							case Types.REAL:
							case Types.SMALLINT:
							case Types.TINYINT:
								arrayFields.add(aux[i]);								
								break;
							}
						}
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		}
		cmbOriginsModel = new DefaultComboBoxModel(arrayLayers);
		cmbDestinationsModel = new DefaultComboBoxModel(arrayLayers);
		cmbOrigins.setModel(cmbOriginsModel);
		cmbDestinations.setModel(cmbDestinationsModel);
		cmbOrigins.setRenderer(new LayerListCellRenderer());
		cmbDestinations.setRenderer(new LayerListCellRenderer());
		
		cmbCostFFModel = new DefaultComboBoxModel(arrayFields);
		cmbServiceLModel = new DefaultComboBoxModel(arrayFields);
		cmbCostFF.setModel(cmbCostFFModel);
		cmbServiceL.setModel(cmbServiceLModel);
	}
	
	private ArrayList<Double> getValuesField(int column) {
		ArrayList<Double> values = new ArrayList<Double>();
		SelectableDataSource dataSource = null;
		try {
			dataSource = currentLyr.getRecordset();
		} catch (ReadDriverException ex){
			ex.printStackTrace();
		}
		long count = this.getRowCount(currentLyr);
		if (count<1) {
			JOptionPane.showMessageDialog((Component) PluginServices.getMDIManager().getActiveWindow(),
					_T("Field_without_values"));
		} else {
			for (int i = 0; i < count; i++) {
				String value = null;
				try {
					value = dataSource.getFieldValue(i,column).getStringValue(ValueWriter.internalValueWriter);
					values.add(Double.parseDouble(value));
				} catch (ReadDriverException ex){
					ex.printStackTrace();
				}
			}
		}
		
		return values;
	}

	private ArrayList<Double> getValuesCostFF(int column) {
		return this.getValuesField(column);
	}

	private ArrayList<Double> getValuesServiceLevel(int column) {
		return this.getValuesField(column);
	}
	
	private long getRowCount(FLyrVect selectedLayer){
		SelectableDataSource dataSource;
		long numberFields = 0;
		try {
			dataSource = selectedLayer.getRecordset();
			numberFields = dataSource.getRowCount();
		} catch (ReadDriverException e) {
			e.printStackTrace();
		}
		return numberFields;
	}

	public FLyrVect getOriginsLayer() {
		return (FLyrVect) cmbOrigins.getSelectedItem();
	}
	
	public FLyrVect getDestinationsLayer() {
		return (FLyrVect) cmbDestinations.getSelectedItem();
	}
	
	public double getTolerance() {
		return Double.parseDouble(txtTolerance.getText());
	}
	
	public ArrayList<Double> getCostFF() {
		return costFF;
	}
	
	public ArrayList<Double> getServiceLevel() {
		return serviceLevel;
	}
	
	public double getEpsilon() {
		return Double.parseDouble(txtEpsilon.getText());		
	}
	
	public String getTripMatrix() {
		return txtTripMatrix.getText();
	}
	
	public String getPathGenerateFile() {
		return txtPathGenerateFile.getText();
	}

	public WindowInfo getWindowInfo() {
		if (wi == null) {
			wi = new WindowInfo(WindowInfo.MODALDIALOG);
			wi.setWidth(500);
			wi.setHeight(500);
			wi.setTitle(_T("Deterministic_car_route_choice"));
		}
		return wi;
	}

	public Object getWindowProfile() {
		return WindowInfo.DIALOG_PROFILE;
	}
	
	private String _T(String str) {
		return PluginServices.getText(this, str);
	}

	private void btnCancelActionPerformed(ActionEvent evt) {
		closeWindow();
	}
	
	private void btnRunActionPerformed(ActionEvent evt) {
		if (getPathGenerateFile().equalsIgnoreCase(""))
		{
			JOptionPane.showMessageDialog((Component) PluginServices.getMDIManager().getActiveWindow(),
					_T("Please_enter_a_valid_created_file"));
			return;
		}
		if (getTripMatrix().equals("")) 
		{
			JOptionPane.showMessageDialog((Component) PluginServices.getMDIManager().getActiveWindow(), 
					_T("Please_enter_a_valid_trip_matrix_file"));
			return;
		}
		try {
			double tol = getTolerance();
		} catch (NumberFormatException e) {
			JOptionPane.showMessageDialog((Component) PluginServices.getMDIManager().getActiveWindow(), 
					_T("Please_enter_a_valid_tolerance_number"));
			return;
		}
		try {
			double eps = getEpsilon();
		} catch (NumberFormatException e) {
			JOptionPane.showMessageDialog((Component) PluginServices.getMDIManager().getActiveWindow(), 
					_T("Please_enter_a_valid_epsilon_number"));
			return;
		}
		
		okPressed = true;
		closeWindow();
	}

	private void closeWindow() {
		if (PluginServices.getMainFrame() !=null) {
			PluginServices.getMDIManager().closeWindow(this);
		}
	}
	
	public boolean isOkPressed() {
		return okPressed;
	}

}
