/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package org.gvsig.graph.mt.solvers;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import javax.swing.JComponent;
import javax.swing.JOptionPane;

import org.gvsig.graph.core.GraphException;
import org.gvsig.graph.core.GvEdge;
import org.gvsig.graph.core.GvFlag;
import org.gvsig.graph.core.IGraph;
import org.gvsig.graph.core.Network;
import org.gvsig.graph.solvers.Route;
import org.gvsig.graph.solvers.ShortestPathSolverDijkstra;

import com.hardcode.gdbms.engine.values.IntValue;
import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.utiles.swing.threads.AbstractMonitorableTask;

/**
 * @author Tonatiuh Miguel (tonatiuh.sit@gmail.com)
 *
 */
public class AssignmentEqTask extends AbstractMonitorableTask{
	
	private Network net;
	private GvFlag[] originFlags;
	private GvFlag[] destinationFlags;
	private ArrayList<Double> ta0;
	private ArrayList<Double> ka;
	private double epsilon;
	private File tripMatrixFile;
	private File generatedFile;
	
	private ArrayList<GvEdge> arcs = new ArrayList<GvEdge>();
	private Route route = null;
	private int numArcs;
	
	private int m;
	private int n;
	private int travelers[][];
	
	private ArrayList<Double> vaKAux = new ArrayList<Double>();
	private ArrayList<Double> taKAux = new ArrayList<Double>();
	private ArrayList<Integer> yaKAux = new ArrayList<Integer>();
	private ArrayList<Double> vaActKAux = new ArrayList<Double>();
	private double thetaKAux;
	
	private int countK;
	private double checkKAux;
	private boolean bExit = false;
	
	private IGraph graph;
	
	public AssignmentEqTask(Network net,
			GvFlag[] originFlags, GvFlag[] destinationFlags,
			ArrayList<Double> costFreeFlow, ArrayList<Double> serviceLevel,
			double epsilon, File tripMatrixFile, File generatedFile){
		this.net = net;
		this.originFlags = originFlags;
		this.destinationFlags = destinationFlags;
		this.ta0 = costFreeFlow;
		this.ka = serviceLevel;
		this.epsilon = epsilon;
		this.tripMatrixFile = tripMatrixFile;
		this.generatedFile = generatedFile;
		
		setInitialStep(0);
		setDeterminatedProcess(false);
		setStatusMessage(PluginServices.getText(this, "Calculating_Assignment_Equilibrium"));
		setFinalStep(arcs.size());
	}

	public void run() throws Exception {
		try {
			
			
			BufferedWriter output = new BufferedWriter(new FileWriter(generatedFile, true));
			
			graph = net.getGraph();
			
//			creo un arreglo de arcos
			for (int i = 0; i < graph.numEdges(); i++) {
				arcs.add(graph.getEdgeByID(i));
			}
//			define el nmero de arcos numArcs
			numArcs = arcs.size();
			
//			define matriz de viajes
			m = originFlags.length;
			n = destinationFlags.length;
			this.travelers = new int [m][n];
			setTravelersOnMatrix();
			
//			inicializa ArrayList con n campos en ceros
			initYaKAux0();
			initArray(taKAux);
			initArray(vaActKAux);
			
//			inicio algoritmo Frank Wolfe
			output.write("++++++++++   Algoritmo Frank Wolfe   ++++++++++");
			output.newLine();
			output.write("Nivel de servicio: "+ka.toString());
			output.newLine();
			output.write("Costo de arco a flujo libre: "+ta0.toString());
			output.newLine();
			
			countK = 0;
			output.write("Paso 1. K= "+countK);
			output.newLine();
			
//			paso 5: calcular ruta mnima y yaKAux
			for (int i = 0; i < originFlags.length; i++) {
				for (int j = 0; j < destinationFlags.length; j++) {
					if (travelers[i][j] == 0)
						continue;
					route = null;
					route = calculateRoute(originFlags[i], destinationFlags[j], net);
					calculateYaKAux(travelers[i][j], route);
				}
			}
			
			countK++;
			
//			paso 6: calcular vaKAux
			vaKAux = castIntToDouble(yaKAux);
			output.write("Paso 6. va"+countK+"="+vaKAux.toString());
			output.newLine();
			
			while (!bExit) {
				output.write("++++++++++   Iteracin "+countK+"   ++++++++++");
				output.newLine();
				
//				paso 7: calcula costo de arco taKAux
				calculateTaKAux();
				output.write("Paso 7. Costo de arco ta"+countK+": "+taKAux.toString());
				output.newLine();
				
//				paso 8: calcula ruta mnima, 9: demanda de la ruta, y 10: demanda de arco yaKAux
				setTaKAuxToNet();
				setYaKAux0();
				for (int i = 0; i < originFlags.length; i++) {
					for (int j = 0; j < destinationFlags.length; j++) {
						if (travelers[i][j] == 0)
							continue;
						route = null;
						route = calculateRoute(originFlags[i], destinationFlags[j], net);
						calculateYaKAux(travelers[i][j], route);
					}
				}
				output.write("Paso 10. Demanda de arco auxiliar ya"+countK+": "+yaKAux.toString());
				output.newLine();
				
//				paso 11: theta
				calculateTheta();
				output.write("Paso 11. Theta"+countK+": "+thetaKAux);
				output.newLine();
				
//				paso 12: demanda actualizada de arco vaActKAux
				calculateVaActKAux();
				output.write("Paso 12. Demanda actualizada de arco va"+
						(countK+1)+": "+vaActKAux.toString());
				output.newLine();
				
//				paso 13: valor de convergencia checkKAux
				calculateCheckKAux();
				output.write("Paso 13. Valor de convergencia: "+checkKAux);
				output.newLine();
				
				if (Math.abs(checkKAux)<=epsilon) {
					bExit = true;
					break;
				}
				
				countK++;
				setVaActToVa();
//				ArrayList en CEROS para nueva iteracin
				setZero(taKAux);
				setZero(vaActKAux);

			}
			output.write("++++++++++   FIN   ++++++++++");
			output.newLine();
			output.flush();
			output.close();
			
		} catch (IOException e) {
			e.printStackTrace();
		} catch (GraphException e) {
			e.printStackTrace();
		}
	}
	
	
//	paso 7: Calcula el Costo del Arco taKAux
	private ArrayList<Double> calculateTaKAux() {
		for (int i = 0; i < numArcs; i++) {
			taKAux.set(i, (ta0.get(i)*(1+0.15*
					(Math.pow((vaKAux.get(i)/ka.get(i)), 4)))));
		}
		return taKAux;
	}
	
//	paso 8: Asigna los Costos de Arco taKAux a la Red
	private void setTaKAuxToNet(){
		for (int i = 0; i < numArcs; i++) {
			arcs.get(i).setDistance(taKAux.get(i));
			arcs.get(i).setWeight(taKAux.get(i));
		}
	}
	
//	paso 9: Calcula la Ruta mnima para cada Origen-Destino
	private Route calculateRoute(GvFlag origen, GvFlag destino, Network net) throws GraphException {
		ShortestPathSolverDijkstra solver = new ShortestPathSolverDijkstra();
		Route route = null;
		try {
			net.removeFlags();
			solver.setNetwork(net);
			net.addFlag(origen);
			net.addFlag(destino);
			String fieldStreetName = (String) net.getLayer().getProperty("network_fieldStreetName");
			solver.setFielStreetName(fieldStreetName);
			route = solver.calculateRoute();
		} catch (GraphException ex) {
			ex.printStackTrace();
		}
		if (route.getFeatureList().size() == 0) {
			JOptionPane.showMessageDialog((JComponent) PluginServices.getMDIManager().getActiveWindow(),
					PluginServices.getText(this, "shortest_path_not_found"));
		}
		return route;
	}
	
//	paso 10: Calcula el valor de Demanda de Arco Auxiliar yaKAux
	private ArrayList<Integer> calculateYaKAux (int travelers_i_j, Route route){
		ArrayList featureList = route.getFeatureList();
		for (int i = 0; i < featureList.size(); i++) {
			int id = 0;
			IFeature feature = (IFeature) featureList.get(i);
			id = ((IntValue)feature.getAttribute(Route.ID_ARC_INDEX)).getValue();
			yaKAux.set(id, (yaKAux.get(id)+travelers_i_j));
		}
		return yaKAux;
	}
	
//	paso 11: Calcula el valor de theta.
	private double calculateTheta(){
		double a = 0;
		double b = 1;
		double eval = 0;
		boolean bEnd = false;
		
		while (!bEnd) {
			thetaKAux = (a+b)/2;
			for (int i = 0; i < numArcs; i++) {
				eval += ((yaKAux.get(i)-vaKAux.get(i))*(ta0.get(i))*
						(1+0.15*(Math.pow(((vaKAux.get(i)+thetaKAux*
								(yaKAux.get(i)-vaKAux.get(i)))/ka.get(i)), 4))));
			}
			if (0<=eval) {
				b = thetaKAux;
			} else {
				a = thetaKAux;
			}
			if (Math.abs(eval)<1) {
				bEnd = true;
				break;
			}
			eval = 0;
		}
		return thetaKAux;
	}
	
//	paso 12: Se calcula la Demanda de Arco Actualizada vaActKAux
	private ArrayList<Double> calculateVaActKAux() {	
		for (int i = 0; i < numArcs; i++) {
			vaActKAux.set(i,(vaKAux.get(i)+thetaKAux*(yaKAux.get(i)-vaKAux.get(i))));
		}
		return vaActKAux;
	}
	
//	paso 13: Estimar el valor de convergencia.
	private double calculateCheckKAux() {
		checkKAux = 0;
		for (int i = 0; i < numArcs; i++) {
			checkKAux +=((yaKAux.get(i)-vaActKAux.get(i))*
					(ta0.get(i))*(1+0.15*(Math.pow((vaActKAux.get(i)/ka.get(i)), 4))));
		}
		return checkKAux;
	}
	
//	Otros mtodos
	private ArrayList<Integer> initYaKAux0() {
		for (int i = 0; i < numArcs; i++) {
			yaKAux.add(i, 0);
		}
		return yaKAux;
	}
	
	private ArrayList<Integer> setYaKAux0() {
		for (int i = 0; i < numArcs; i++) {
			yaKAux.set(i, 0);
		}
		return yaKAux;
	}
	
	private ArrayList<Double> castIntToDouble(ArrayList<Integer> arrInteger){
		ArrayList<Double> arrDouble = new ArrayList<Double>();
		for (int i = 0; i < arrInteger.size(); i++) {
			arrDouble.add((double)arrInteger.get(i));
		}
		return arrDouble;
	}
	
	private ArrayList<Double> initArray(ArrayList<Double> arrayEmpty) {
		for (int i = 0; i < numArcs; i++) {
			arrayEmpty.add(i, 0d);
		}
		return arrayEmpty;
	}
	
	private ArrayList<Double> setZero(ArrayList<Double> arrayData) {
		for (int i = 0; i < numArcs; i++) {
			arrayData.set(i, 0d);
		}
		return arrayData;
	}
	
	private ArrayList<Double> setVaActToVa(){
		for (int i = 0; i < numArcs; i++) {
			vaKAux.set(i, vaActKAux.get(i));
		}
		return vaKAux;
	}
	
//	Mtodo que lee el archivo de viajeros y los pasa a una matriz
	private int [][] setTravelersOnMatrix() {
		BufferedReader input = null;
		String linea = null;
		try {
			input = new BufferedReader(new FileReader(tripMatrixFile));
			while ((linea=input.readLine()) != null) {
				int fila;
				int columna;
				String[] numero = linea.split("\t");
				fila = Integer.parseInt(numero[0]);
				columna = Integer.parseInt(numero[1]);
				travelers[fila][columna] = Integer.parseInt(numero[2]);
			}
			input.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		return travelers;
	}
	
}

